==> ogmrip_0.11.1.orig.tar.gz: ogmrip-0.11.1/libogmrip/ogmrip-mp3.c <==
/* OGMRip - A library for DVD ripping and encoding
 * Copyright (C) 2004-2007 Olivier Rolland <billl@users.sf.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "ogmrip-mp3.h"
#include "ogmrip-mplayer.h"
#include "ogmrip-plugin.h"
#include "ogmrip-fs.h"

#include "ogmjob-pipeline.h"
#include "ogmjob-exec.h"

#include <unistd.h>
#include <glib/gstdio.h>
#include <glib/gi18n-lib.h>

#define MP3_SPF 576

static gint ogmrip_mp3_run (OGMJobSpawn *spawn);
static gint ogmrip_mp3_get_samples_per_frame (OGMRipAudio *audio);

static gchar **
ogmrip_mp3_command (OGMRipAudio *audio, const gchar *input, const gchar *output)
{
  static gchar *presets[][2] =
  {
    { "96",       NULL },
    { "112",      NULL },
    { "128",      NULL },
    { "fast",    "medium" },
    { "medium",   NULL },
    { "fast",    "standard" },
    { "standard", NULL },
    { "fast",    "extreme" },
    { "extreme",  NULL },
    { "fast",    "insane" },
    { "insane",   NULL }
  };

  GPtrArray *argv;
  gint quality;

  g_return_val_if_fail (OGMRIP_IS_AUDIO (audio), NULL);
  g_return_val_if_fail (input != NULL, NULL);

  if (!output)
    output = ogmrip_codec_get_output (OGMRIP_CODEC (audio));
  g_return_val_if_fail (output != NULL, NULL);

  quality = ogmrip_audio_get_quality (audio);

  argv = g_ptr_array_new ();
  g_ptr_array_add (argv, g_strdup ("lame"));
  g_ptr_array_add (argv, g_strdup ("--nohist"));
  g_ptr_array_add (argv, g_strdup ("-h"));
/*
  g_ptr_array_add (argv, g_strdup ("-t"));
*/
  g_ptr_array_add (argv, g_strdup ("-r"));
  g_ptr_array_add (argv, g_strdup ("-x"));
  g_ptr_array_add (argv, g_strdup ("-s"));
  g_ptr_array_add (argv, g_strdup_printf ("%.1f", ogmrip_audio_get_sample_rate (audio) / 1000.0));
  g_ptr_array_add (argv, g_strdup ("--preset"));
  g_ptr_array_add (argv, g_strdup (presets[quality][0]));
  if (presets[quality][1])
    g_ptr_array_add (argv, g_strdup (presets[quality][1]));
  g_ptr_array_add (argv, g_strdup (input));
  g_ptr_array_add (argv, g_strdup (output));
  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}

static gchar **
ogmrip_wav_command (OGMRipAudio *audio, gboolean header, const gchar *input, const gchar *output)
{
  GPtrArray *argv;

  argv = ogmrip_mplayer_wav_command (audio, FALSE, NULL, output);
  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}

G_DEFINE_TYPE (OGMRipMp3, ogmrip_mp3, OGMRIP_TYPE_AUDIO)

static void
ogmrip_mp3_class_init (OGMRipMp3Class *klass)
{
  OGMJobSpawnClass *spawn_class;
  OGMRipAudioClass *audio_class;

  spawn_class = OGMJOB_SPAWN_CLASS (klass);
  spawn_class->run = ogmrip_mp3_run;

  audio_class = OGMRIP_AUDIO_CLASS (klass);
  audio_class->get_samples_per_frame = ogmrip_mp3_get_samples_per_frame;
}

static void
ogmrip_mp3_init (OGMRipMp3 *mp3)
{
}

static gint
ogmrip_mp3_run (OGMJobSpawn *spawn)
{
  OGMJobSpawn *pipeline;
  OGMJobSpawn *child;
  gchar **argv, *fifo;
  gint result;

  result = OGMJOB_RESULT_ERROR;

  fifo = ogmrip_fs_mkftemp ("fifo.XXXXXX", NULL);

  pipeline = ogmjob_pipeline_new ();
  ogmjob_container_add (OGMJOB_CONTAINER (spawn), pipeline);
  g_object_unref (pipeline);

  argv = ogmrip_wav_command (OGMRIP_AUDIO (spawn), FALSE, NULL, fifo);
  if (argv)
  {
    child = ogmjob_exec_newv (argv);
    ogmjob_exec_add_watch_full (OGMJOB_EXEC (child), (OGMJobWatch) ogmrip_mplayer_wav_watch, spawn, TRUE, FALSE, FALSE);
    ogmjob_container_add (OGMJOB_CONTAINER (pipeline), child);
    g_object_unref (child);

    argv = ogmrip_mp3_command (OGMRIP_AUDIO (spawn), fifo, NULL);
    if (argv)
    {
      child = ogmjob_exec_newv (argv);
      ogmjob_container_add (OGMJOB_CONTAINER (pipeline), child);
      g_object_unref (child);

      result = OGMJOB_SPAWN_CLASS (ogmrip_mp3_parent_class)->run (spawn);
    }
  }

  ogmjob_container_remove (OGMJOB_CONTAINER (spawn), pipeline);

  if (g_file_test (fifo, G_FILE_TEST_EXISTS))
    g_unlink (fifo);
  g_free (fifo);

  return result;
}

static gint
ogmrip_mp3_get_samples_per_frame (OGMRipAudio *audio)
{
  return MP3_SPF;
}

/**
 * ogmrip_mp3_new:
 * @audio: An #OGMDvdAudioStream
 * @output: The output file
 *
 * Creates a new #OGMRipMp3.
 *
 * Returns: The new #OGMRipMp3
 */
OGMJobSpawn *
ogmrip_mp3_new (OGMDvdAudioStream *audio, const gchar *output)
{
  g_return_val_if_fail (audio != NULL, NULL);
  g_return_val_if_fail (output && *output, NULL);

  return g_object_new (OGMRIP_TYPE_MP3, "stream", audio, "output", output, NULL);
}

static OGMRipPluginAudioCodec mp3_plugin =
{
  NULL,
  G_TYPE_NONE,
  "mp3",
  N_("MPEG-1 layer III (MP3)"),
  OGMRIP_FORMAT_MP3
};

OGMRipPluginAudioCodec *
ogmrip_init_plugin (void)
{
  mp3_plugin.type = OGMRIP_TYPE_MP3;

  return &mp3_plugin;
}

==> ogmrip_0.12.2.orig.tar.gz: ogmrip-0.12.2/libogmrip/ogmrip-mp3.c <==
/* OGMRip - A library for DVD ripping and encoding
 * Copyright (C) 2004-2008 Olivier Rolland <billl@users.sf.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "ogmrip-mp3.h"
#include "ogmrip-mplayer.h"
#include "ogmrip-plugin.h"
#include "ogmrip-fs.h"
#include "ogmrip-version.h"

#include "ogmjob-pipeline.h"
#include "ogmjob-exec.h"

#include <unistd.h>
#include <glib/gstdio.h>
#include <glib/gi18n-lib.h>

#define PROGRAM "lame"
#define MP3_SPF 1152

static gint ogmrip_mp3_run (OGMJobSpawn *spawn);
static gint ogmrip_mp3_get_samples_per_frame (OGMRipAudioCodec *audio);

static gchar **
ogmrip_mp3_command (OGMRipAudioCodec *audio, gboolean header, const gchar *input, const gchar *output)
{
  static gchar *presets[][2] =
  {
    { "96",       NULL },
    { "112",      NULL },
    { "128",      NULL },
    { "fast",    "medium" },
    { "medium",   NULL },
    { "fast",    "standard" },
    { "standard", NULL },
    { "fast",    "extreme" },
    { "extreme",  NULL },
    { "fast",    "insane" },
    { "insane",   NULL }
  };

  GPtrArray *argv;
  gint quality;

  g_return_val_if_fail (OGMRIP_IS_AUDIO_CODEC (audio), NULL);
  g_return_val_if_fail (input != NULL, NULL);

  if (!output)
    output = ogmrip_codec_get_output (OGMRIP_CODEC (audio));
  g_return_val_if_fail (output != NULL, NULL);

  quality = ogmrip_audio_codec_get_quality (audio);

  argv = g_ptr_array_new ();
  g_ptr_array_add (argv, g_strdup (PROGRAM));
  g_ptr_array_add (argv, g_strdup ("--nohist"));
  g_ptr_array_add (argv, g_strdup ("-h"));

  if (!header)
  {
    g_ptr_array_add (argv, g_strdup ("-r"));
    g_ptr_array_add (argv, g_strdup ("-x"));
    g_ptr_array_add (argv, g_strdup ("-s"));
    g_ptr_array_add (argv, g_strdup_printf ("%.1f",
          ogmrip_audio_codec_get_sample_rate (audio) / 1000.0));
  }

  g_ptr_array_add (argv, g_strdup ("--preset"));
  g_ptr_array_add (argv, g_strdup (presets[quality][0]));
  if (presets[quality][1])
    g_ptr_array_add (argv, g_strdup (presets[quality][1]));
  g_ptr_array_add (argv, g_strdup (input));
  g_ptr_array_add (argv, g_strdup (output));
  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}

static gchar **
ogmrip_wav_command (OGMRipAudioCodec *audio, gboolean header, const gchar *input, const gchar *output)
{
  GPtrArray *argv;

  argv = ogmrip_mplayer_wav_command (audio, header, output);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}

G_DEFINE_TYPE (OGMRipMp3, ogmrip_mp3, OGMRIP_TYPE_AUDIO_CODEC)

static void
ogmrip_mp3_class_init (OGMRipMp3Class *klass)
{
  OGMJobSpawnClass *spawn_class;
  OGMRipAudioCodecClass *audio_class;

  spawn_class = OGMJOB_SPAWN_CLASS (klass);
  spawn_class->run = ogmrip_mp3_run;

  audio_class = OGMRIP_AUDIO_CODEC_CLASS (klass);
  audio_class->get_samples_per_frame = ogmrip_mp3_get_samples_per_frame;
}

static void
ogmrip_mp3_init (OGMRipMp3 *mp3)
{
}

static gint
ogmrip_mp3_run (OGMJobSpawn *spawn)
{
  GError *error = NULL;
  OGMJobSpawn *pipeline;
  OGMJobSpawn *child;
  gchar **argv, *fifo;
  gint result;

  result = OGMJOB_RESULT_ERROR;

  fifo = ogmrip_fs_mkftemp ("fifo.XXXXXX", &error);
  if (!fifo)
  {
    ogmjob_spawn_propagate_error (spawn, error);
    return OGMJOB_RESULT_ERROR;
  }

  pipeline = ogmjob_pipeline_new ();
  ogmjob_container_add (OGMJOB_CONTAINER (spawn), pipeline);
  g_object_unref (pipeline);

  argv = ogmrip_wav_command (OGMRIP_AUDIO_CODEC (spawn), TRUE, NULL, fifo);
  if (argv)
  {
    child = ogmjob_exec_newv (argv);
    ogmjob_exec_add_watch_full (OGMJOB_EXEC (child),
        (OGMJobWatch) ogmrip_mplayer_wav_watch, spawn, TRUE, FALSE, FALSE);
    ogmjob_container_add (OGMJOB_CONTAINER (pipeline), child);
    g_object_unref (child);

    argv = ogmrip_mp3_command (OGMRIP_AUDIO_CODEC (spawn), TRUE, fifo, NULL);
    if (argv)
    {
      child = ogmjob_exec_newv (argv);
      ogmjob_container_add (OGMJOB_CONTAINER (pipeline), child);
      g_object_unref (child);

      result = OGMJOB_SPAWN_CLASS (ogmrip_mp3_parent_class)->run (spawn);
    }
  }

  ogmjob_container_remove (OGMJOB_CONTAINER (spawn), pipeline);

  if (g_file_test (fifo, G_FILE_TEST_EXISTS))
    g_unlink (fifo);
  g_free (fifo);

  return result;
}

static gint
ogmrip_mp3_get_samples_per_frame (OGMRipAudioCodec *audio)
{
  return MP3_SPF;
}

/**
 * ogmrip_mp3_new:
 * @audio: An #OGMDvdAudioStream
 * @output: The output file
 *
 * Creates a new #OGMRipMp3.
 *
 * Returns: The new #OGMRipMp3
 */
OGMJobSpawn *
ogmrip_mp3_new (OGMDvdAudioStream *audio, const gchar *output)
{
  g_return_val_if_fail (audio != NULL, NULL);
  g_return_val_if_fail (output && *output, NULL);

  return g_object_new (OGMRIP_TYPE_MP3, "stream", audio, "output", output, NULL);
}

static OGMRipAudioPlugin mp3_plugin =
{
  NULL,
  G_TYPE_NONE,
  "mp3",
  N_("MPEG-1 layer III (MP3)"),
  OGMRIP_FORMAT_MP3
};

OGMRipAudioPlugin *
ogmrip_init_plugin (void)
{
  gchar *fullname;

  if (!ogmrip_check_mplayer ())
    return NULL;

  fullname = g_find_program_in_path (PROGRAM);
  if (!fullname)
    return NULL;
  g_free (fullname);

  mp3_plugin.type = OGMRIP_TYPE_MP3;

  return &mp3_plugin;
}
